Imports System
Imports System.ComponentModel
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports AjaxControlToolkit
Imports System.Security.Cryptography

#Region "Assembly Resource Attribute"
<Assembly: System.Web.UI.WebResource("DPRAjaxControlToolkit.RSAEncryptBehavior.js", "text/javascript")> 
<Assembly: System.Web.UI.WebResource("DPRAjaxControlToolkit.Barrett.js", "text/javascript")> 
<Assembly: System.Web.UI.WebResource("DPRAjaxControlToolkit.BigInt.js", "text/javascript")> 
<Assembly: System.Web.UI.WebResource("DPRAjaxControlToolkit.Funcoes.js", "text/javascript")> 
<Assembly: System.Web.UI.WebResource("DPRAjaxControlToolkit.RSA.js", "text/javascript")> 
#End Region

Namespace DPRAjaxControlToolkit

    <Designer(GetType(RSAEncryptDesigner))> _
    <ClientScriptResource("DPRAjaxControlToolkit.RSAEncryptBehavior", "DPRAjaxControlToolkit.RSAEncryptBehavior.js")> _
    <TargetControlType(GetType(TextBox))> _
    Public Class RSAEncryptExtender
        Inherits ExtenderControlBase

        'Public Sub New(ByVal provider As RSACryptoServiceProvider)
        Public Sub New()
            'Me.RSACryptoServiceProvider = provider
            MyBase.New()
            Me.EnableClientState = True
        End Sub

#Region "Atributos"
        'Private rsa As New RSACrypto()
#End Region

#Region "Proprieades"
        Private _RSACryptoServiceProvider As RSACryptoServiceProvider
        <RequiredProperty()> _
        Public Property RSACryptoServiceProvider() As RSACryptoServiceProvider
            Get
                Return _RSACryptoServiceProvider
            End Get
            Set(ByVal value As RSACryptoServiceProvider)
                _RSACryptoServiceProvider = value
            End Set
        End Property

        Private ReadOnly Property RSAParameters() As RSAParameters
            Get
                If Me.RSACryptoServiceProvider IsNot Nothing Then
                    Return Me.RSACryptoServiceProvider.ExportParameters(True)
                Else
                    Return Nothing
                End If
            End Get
        End Property
#End Region

#Region "Proprieades AJAX"
        <ExtenderControlProperty()> _
        <DefaultValue("")> _
        Public Property Exponent() As String
            Get
                Return GetPropertyValue("Exponent", String.Empty)
            End Get
            Set(ByVal value As String)
                SetPropertyValue("Exponent", value)
            End Set
        End Property

        <ExtenderControlProperty()> _
        <DefaultValue("")> _
        Public Property Modulus() As String
            Get
                Return GetPropertyValue("Modulus", String.Empty)
            End Get
            Set(ByVal value As String)
                SetPropertyValue("Modulus", value)
            End Set
        End Property

        <ExtenderControlProperty()> _
        <DefaultValue("")> _
        Public Property ClientChallenge() As String
            Get
                Return GetPropertyValue("ClientChallenge", String.Empty)
            End Get
            Set(ByVal value As String)
                SetPropertyValue("ClientChallenge", value)
            End Set
        End Property

        <ExtenderControlProperty()> _
        <DefaultValue(False)> _
        Public Property HasChallenge() As Boolean
            Get
                Return GetPropertyValue(Of Boolean)("HasChallenge", False)
            End Get
            Set(ByVal value As Boolean)
                SetPropertyValue(Of Boolean)("HasChallenge", value)
            End Set
        End Property


        <ExtenderControlProperty(True)> _
        <DefaultValue("")> _
        Public Property EncryptedValue() As String
            Get
                Return GetPropertyValue("EncryptedValue", String.Empty)
            End Get
            Set(ByVal value As String)
                SetPropertyValue("EncryptedValue", value)
            End Set
        End Property

#End Region

#Region "Challange"
        Protected Function GetChallenge() As String
            Dim challenge As String = Me.ClientChallenge

            If String.IsNullOrEmpty(challenge) Then
                Return CreateChallengeString()
            Else
                ' Check if the server counterpart exist or not expired
                Dim rawSalt As Byte() = System.Web.HttpContext.Current.Cache(challenge)
                If rawSalt Is Nothing Then
                    Return CreateChallengeString()
                Else
                    Return challenge
                End If
            End If
        End Function

        Private Function CreateChallengeString() As String
            Dim rng As New Random(DateTime.Now.Millisecond)

            ' Create random string
            Dim salt(64) As Byte
            For i As Integer = 0 To 64 - 1
                salt(i) = CType(rng.Next(65, 90), Byte) ' a-z
                salt(i) = CType(rng.Next(97, 122), Byte) ' A-Z
            Next

            Dim challenge As String = ComputeHashString(salt, Me.RSAParameters.D)

            System.Web.HttpContext.Current.Cache.Insert(challenge, salt, Nothing, DateTime.Now.AddMinutes(20), System.Web.Caching.Cache.NoSlidingExpiration)

            Return challenge
        End Function

        Private Function ComputeHashString(ByVal salt As Byte(), ByVal uniqueKey As Byte()) As String
            ' Concat before hashing
            Dim target(salt.Length + uniqueKey.Length) As Byte
            System.Buffer.BlockCopy(salt, 0, target, 0, salt.Length)
            System.Buffer.BlockCopy(uniqueKey, 0, target, salt.Length, uniqueKey.Length)

            Dim sha As New SHA1Managed()
            Return StringHelper.ToBase64(sha.ComputeHash(target))
        End Function

        Private Sub RemoveChallengeString(ByVal challenge As String)
            System.Web.HttpContext.Current.Cache.Remove(challenge)
        End Sub

#End Region

        Private Sub RSAEncryptExtender_ClientStateValuesLoaded(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.ClientStateValuesLoaded
            If Not String.IsNullOrEmpty(Me.ClientState) Then
                Dim valorBase64 As String = String.Empty
                Dim valor As String = String.Empty

                Dim rsa As New RSACrypto(Me.RSACryptoServiceProvider)
                Dim tmp As String = StringHelper.ASCIIBytesToString(rsa.Decrypt(StringHelper.HexStringToBytes(Me.ClientState)))

                If Me.HasChallenge Then
                    Dim challenge As String = String.Empty
                    Dim parts() As String = tmp.Split("\")

                    If parts.Length = 2 Then
                        challenge = parts(0)

                        If Not String.IsNullOrEmpty(challenge) And challenge = Me.GetChallenge() Then
                            valorBase64 = parts(1)
                        Else
                            Throw New ApplicationException("Challenge invlido.")
                        End If
                    End If

                    RemoveChallengeString(challenge)

                Else
                    valorBase64 = tmp
                End If

                valor = StringHelper.UTF7BytesToString(StringHelper.FromBase64(valorBase64))

                CType(Me.TargetControl, TextBox).Text = valor

            End If

        End Sub

        Private Sub RSAEncryptExtender_Init(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Init

            ScriptManager.RegisterClientScriptResource(Me.Page, GetType(RSAEncryptExtender), "DPRAjaxControlToolkit.Barrett.js")
            ScriptManager.RegisterClientScriptResource(Me.Page, GetType(RSAEncryptExtender), "DPRAjaxControlToolkit.BigInt.js")
            ScriptManager.RegisterClientScriptResource(Me.Page, GetType(RSAEncryptExtender), "DPRAjaxControlToolkit.Funcoes.js")
            ScriptManager.RegisterClientScriptResource(Me.Page, GetType(RSAEncryptExtender), "DPRAjaxControlToolkit.RSA.js")

        End Sub

        Private Sub RSAEncryptExtender_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
            If Me.RSACryptoServiceProvider Is Nothing Then
                Throw New ApplicationException("O RSACryptoServiceProvider para " & Me.ClientID & "  obrigatrio.")
            End If
        End Sub

        Private Sub RSAEncryptExtender_PreRender(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.PreRender
            SetPropertyValue("Exponent", StringHelper.BytesToHexString(Me.RSAParameters.Exponent))
            SetPropertyValue("Modulus", StringHelper.BytesToHexString(Me.RSAParameters.Modulus))
            SetPropertyValue("ClientChallenge", Me.GetChallenge())
            'SetPropertyValue("EncryptedValue", Me.EncryptedValue())
        End Sub

    End Class

End Namespace
